/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <getopt.h>
#include <unistd.h>
#include <math.h>

#include "aubio.h"	

#include "utils.h"

/* command line arguments (from utils.h) */
const char * input_filename = NULL;
const char * output_filename = NULL; /* unused */
int verbose = 0; int usejack = 0;

/* the processing function */
int aubio_process(float **input, float **output, int nframes);

/* settings */
aubio_onsetdetection_type type_onset = hfc; 
      /* energy,specdiff,hfc,complexdomain,phase */
smpl_t threshold = 0.1;

/* default settings */
int buffer_size = 1024;
int overlap_size = 512;
int channels = 1;
int samplerate = 44100;

/* global objects */
aubio_pvoc_t * pv_onset;
aubio_pvoc_t * pv_pitch;
aubio_filter_t * filter;

fvec_t * ibuf;
fvec_t * obuf;
cvec_t * fftgrain_onset;
cvec_t * fftgrain_pitch;
	
//fvec_t * woodblock;

aubio_onsetdetection_t *o;
fvec_t *onset; /* onset results */
pickparams_t * parms; /* onset pick picker parameters */

aubio_pitchdetection_t * p; /* the pitch detection */

smpl_t pitch[6]; /* the onset detection function and the  candidadtes */

int aubio_process(float **input, float **output, int nframes) {
	unsigned int i; 			/*channels*/
	unsigned int j; 			/*frames*/   
	unsigned int pos = 0; /*frames%dspblocksize*/
	//unsigned int ons = 0; /*onset counter*/
	for (j=0;j<nframes;j++) {
		
		for (i=0;i<channels;i++) { 
			/* write input to datanew */
			fvec_write_sample(ibuf, input[i][j], i, pos);
			/* put synthnew in output */
			output[i][j] = fvec_read_sample(obuf, i, pos);
		}
		/*time for fft*/
		if (pos == overlap_size-1) {
      /* onset detection */
      aubio_pvoc_do (pv_onset,ibuf, fftgrain_onset);
      aubio_onsetdetection(o,fftgrain_onset, onset);
      //isonset = aubio_peakpick_pimrt(onset,parms);
      /* pitch detection */
      aubio_filter_do(filter,ibuf);
      aubio_pvoc_do (pv_pitch,ibuf, fftgrain_pitch);
      //pitch = aubio_pitchdetect(p,fftgrain_pitch);
      aubio_pitch_cands(p,fftgrain_pitch, pitch);
			pos = -1; /* so it will be zero next j loop */
		}
		pos++;
	}
	return 1;
}

int main (int argc, char **argv) {  
	int frames;
	aubio_file_t * file = NULL;
  //aubio_file_t * onsetfile = new_file_ro(onset_filename);
	parse_args(argc, argv);

	if(!usejack) 
	{
		debug("Opening  file (ro) ...\n");
		file = new_file_ro (input_filename);
		file_info(file);
		channels = file_channels(file);
	}

	ibuf = new_fvec(overlap_size, channels);
	obuf = new_fvec(overlap_size, channels);
	fftgrain_onset = new_cvec(buffer_size, channels);
	fftgrain_pitch = new_cvec(buffer_size*4, channels);
	
	/* phase vocoder */
	pv_onset = new_aubio_pvoc( buffer_size, overlap_size, channels);
	pv_pitch = new_aubio_pvoc(4*buffer_size, overlap_size, channels);
  filter = new_aubio_adsgn_filter((smpl_t)samplerate);

	p = new_aubio_pitchdetection(buffer_size,channels);
	parms = new_aubio_peakpicker(threshold);
	
	/* onset init */
  debug("type_onset: %d\n", (int)type_onset);
  debug("threshold : %f\n", threshold);
	o = new_aubio_onsetdetection(type_onset,buffer_size,channels);
	onset = new_fvec(overlap_size, channels);
	// command line argument parsing
	if(usejack) {
		aubio_jack_t * jack_setup;
		debug("Jack init ...\n");
		jack_setup = new_aubio_jack(channels, channels, (aubio_process_func_t)aubio_process);

    aubio_pvoc_do (pv_onset,ibuf, fftgrain_onset);

		debug("Jack activation ...\n");
		aubio_jack_activate(jack_setup);

		debug("Processing (Ctrl+C to quit) ...\n");
		pause();

		aubio_jack_close(jack_setup);

	} else {
		fvec_t * inputbuf ;
		fvec_t * outputbuf;
		/* file objects */

		inputbuf 	= new_fvec(overlap_size, channels);
		outputbuf	= new_fvec(overlap_size, channels);	
		/* phasevoc init */

    /* initialise fftgrain */
    for (frames=0;frames<4;frames++) {
      aubio_pvoc_do (pv_onset,inputbuf, fftgrain_onset);
      aubio_pvoc_do (pv_pitch,inputbuf, fftgrain_pitch);
    }
  
    for (frames=0;frames<1;frames++) {
      file_read(file, overlap_size, inputbuf);
      aubio_pvoc_do (pv_onset,inputbuf, fftgrain_onset);
      aubio_filter_do(filter,inputbuf);
      aubio_pvoc_do (pv_pitch,inputbuf, fftgrain_pitch);
    }
		
    debug("Processing ...\n");
    frames = 0;
    while ((overlap_size == file_read(file, overlap_size, inputbuf)) ) 
    {
			//isonset=0;
			aubio_process(inputbuf->data, outputbuf->data, overlap_size);
      /*
      for (k=0;k<5;k++){
        if (pitch[k]>1)
          pitch[k] = (bintomidi(pitch[k],samplerate,buffer_size*4));
        else
          pitch[k] = 1;
      }*/
      /*output onset detection function, pitch energies fo/1,2,3,4,5 and 
       * comb fo midipitch for each frame*/
      debug("%f\t%f\t%f\t%f\t%f\t%f\t%f\n",onset->data[0][0],
         pitch[0],pitch[1],pitch[2],pitch[3],pitch[4], 
         (pitch[5]>0.)?bintomidi(pitch[5],samplerate,buffer_size*4):0.);
			frames++;
		}
    debug("Processed %d frames of %d samples.\n", frames, buffer_size);
		del_file(file);
	}

	del_aubio_pvoc(pv_onset);
	del_aubio_pvoc(pv_pitch);
	del_fvec(obuf);
	del_fvec(ibuf);
	del_cvec(fftgrain_onset);
	del_cvec(fftgrain_pitch);
	del_fvec(onset);
	
	debug("End of program.\n");
	fflush(stderr);
	return 0;
}

